==> workers_ai_provider-1.0.0.tar.gz: workers_ai_provider/src/Plugin/AiProvider/WorkersAiProvider.php <==
<?php

namespace Drupal\workers_ai_provider\Plugin\AiProvider;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\AiProvider;
use Drupal\ai\Base\AiProviderClientBase;
use Drupal\ai\Enum\AiModelCapability;
use Drupal\ai\Exception\AiRateLimitException;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatInterface;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\ai\OperationType\Chat\ChatOutput;
use Drupal\ai\Traits\OperationType\ChatTrait;
use OpenAI\Client;
use Symfony\Component\Yaml\Yaml;

/**
 * Plugin implementation of the 'workers ai' provider.
 */
#[AiProvider(
  id: 'workers_ai',
  label: new TranslatableMarkup('Workers AI'),
)]
class WorkersAiProvider extends AiProviderClientBase implements
  ContainerFactoryPluginInterface,
  ChatInterface {

  use ChatTrait;

  /**
   * The OpenAI Client for API calls.
   *
   * @var \OpenAI\Client|null
   */
  protected $client;

  /**
   * API Key.
   *
   * @var string
   */
  protected string $apiKey = '';

  /**
   * API Key.
   *
   * @var string
   */
  protected string $cloudfareBaseURL = '';

  /**
   * API Key.
   *
   * @var string
   */
  protected string $cloudfareAccountId = '';

  /**
   * {@inheritdoc}
   */
  public function getConfiguredModels(?string $operation_type = NULL, array $capabilities = []): array {
    // No vision support.
    $cloudfareBaseURL = $this->getConfig()->get('cloudfare_base_url');
    $cloudfareAccountId = $this->getConfig()->get('cloudfare_account_id');
    $base_url = $cloudfareBaseURL . '/accounts/' . $cloudfareAccountId;
    if (in_array(AiModelCapability::ChatWithImageVision, $capabilities)) {
      return [];
    }
    try {
      $client = \Drupal::httpClient();
      $response = $client->request('GET', $base_url. "/ai/models/search", [
        'headers' => [
          'Authorization' => "Bearer pKum0EeqkErDqfIBwqDThOAkxRloUGHH5Wg7GMXI",
          'Content-Type' => 'application/json',
        ],
      ]);
    
      $body = $response->getBody()->getContents();
      $response_data = json_decode($body, TRUE);

    } catch (\Exception $e) {
      \Drupal::logger('workers_ai')->error('API request failed: @message', ['@message' => $e->getMessage()]);
      return [];
    }

    $models = [];
    $model_names =[];
    if (isset($response_data['result'])) {
      foreach ($response_data['result'] as $model) {
        $models[$model['name']] = $model['name'];
      }
    }

    if (in_array(AiModelCapability::ChatJsonOutput, $capabilities)) {
      foreach ($models as $model_id => $model_name) {
        if (strpos($model_id, 'llama') === FALSE) {
          unset($models[$model_id]);
        }
      }
    }
    return $models;
  }

  /**
   * {@inheritdoc}
   */
  public function isUsable(?string $operation_type = NULL, array $capabilities = []): bool {
    // No vision support.
    if (in_array(AiModelCapability::ChatWithImageVision, $capabilities)) {
      return FALSE;
    }
    // If its not configured, it is not usable.
    if (!$this->getConfig()->get('api_key')) {
      return FALSE;
    }
    else{
      $this->apiKey = $this->getConfig()->get('api_key');
    }
    if (!$this->getConfig()->get('cloudfare_base_url')) {
      return FALSE;
    }
    else{
      $this->cloudfareBaseURL = $this->getConfig()->get('cloudfare_base_url');
    }
    if (!$this->getConfig()->get('cloudfare_account_id')) {
      return FALSE;
    }
    else{
      $this->cloudfareAccountId = $this->getConfig()->get('cloudfare_account_id');
    }
    // If its one of the bundles that Workers AI supports its usable.
    if ($operation_type) {
      return in_array($operation_type, $this->getSupportedOperationTypes());
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedOperationTypes(): array {
    return [
      'chat',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(): ImmutableConfig {
    return $this->configFactory->get('workers_ai_provider.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function getApiDefinition(): array {
    // Load the configuration.
    $definition = Yaml::parseFile($this->moduleHandler->getModule('workers_ai_provider')->getPath() . '/definitions/api_defaults.yml');
    return $definition;
  }

  /**
   * {@inheritdoc}
   */
  public function getModelSettings(string $model_id, array $generalConfig = []): array {
    return $generalConfig;
  }

  /**
   * {@inheritdoc}
   */
  public function setAuthentication(mixed $authentication): void {
    // Set the new API key and reset the client.
    $this->apiKey = $authentication;
    $this->client = NULL;
  }

  /**
   * Gets the raw client.
   *
   * This is the client for inference.
   *
   * @return \OpenAI\Client
   *   The OpenAI client.
   */
  public function getClient(): Client {
    $this->loadClient();
    return $this->client;
  }

  /**
   * Loads the Workers AI Client with authentication if not initialized.
   */
  protected function loadClient(): void {
    if (!$this->client) {
      if (!$this->apiKey) {
        $this->setAuthentication($this->loadApiKey());
      }
      $cloudfareBaseURL = $this->getConfig()->get('cloudfare_base_url');
      $cloudfareAccountId = $this->getConfig()->get('cloudfare_account_id'); 
      $final_base_url = $cloudfareBaseURL .'/accounts/'. $cloudfareAccountId;
      $this->client = \OpenAI::factory()
        ->withApiKey($this->apiKey)
        ->withBaseUri($final_base_url)
        ->withHttpClient($this->httpClient)
        ->make();       
    }
  }

  /**
   * {@inheritdoc}
   */
  public function chat(array|string|ChatInput $input, string $model_id, array $tags = []): ChatOutput {
    $cloudfareBaseURL = $this->getConfig()->get('cloudfare_base_url');
    $cloudfareAccountId = $this->getConfig()->get('cloudfare_account_id');
    $base_url = $cloudfareBaseURL . '/accounts/' . $cloudfareAccountId;
    $chat_input = [];
  
    // Prepare chat messages.
    if ($input instanceof ChatInput) {
      if ($this->chatSystemRole) {
        $chat_input[] = [
          'role' => 'system',
          'content' => $this->chatSystemRole,
        ];
      }
      foreach ($input->getMessages() as $message) {
        $chat_input[] = [
          'role' => 'user',
          'content' => $message->getText(),
        ];
      }
    }
    // Build API request payload.
    $payload = [
      'messages' => $chat_input,
    ];
  
    try {
      $client = \Drupal::httpClient();
      $response = $client->post(
        $base_url."/ai/run/{$model_id}",
        [
          'headers' => [
            'Authorization' => 'Bearer pKum0EeqkErDqfIBwqDThOAkxRloUGHH5Wg7GMXI',
            'Content-Type' => 'application/json',
          ],
          'json' => $payload,
        ]
      );
  
      $data = json_decode($response->getBody()->getContents(), TRUE);
      $message_text = $data['result']['response'] ?? 'No response';  
      $message = new ChatMessage('assistant', $message_text);
      return new ChatOutput($message, $data, []);
    }
    catch (\Exception $e) {
      if (strpos($e->getMessage(), 'Rate limit') !== FALSE) {
        throw new AiRateLimitException($e->getMessage());
      }
      throw $e;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSetupData(): array {
    return [
      'key_config_name' => 'api_key',
      'default_models' => [
        'chat' => '@cf/meta/llama-3.3-70b-instruct-fp8-fast',
      ],
    ];
  }

}

==> workers_ai_provider-1.0.1.tar.gz: workers_ai_provider/src/Plugin/AiProvider/WorkersAiProvider.php <==
<?php

namespace Drupal\workers_ai_provider\Plugin\AiProvider;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\AiProvider;
use Drupal\ai\Base\AiProviderClientBase;
use Drupal\ai\Enum\AiModelCapability;
use Drupal\ai\Exception\AiRateLimitException;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatInterface;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\ai\OperationType\Chat\ChatOutput;
use Drupal\ai\Traits\OperationType\ChatTrait;
use OpenAI\Client;
use Symfony\Component\Yaml\Yaml;


/**
 * Plugin implementation of the 'workers ai' provider.
 */
#[AiProvider(
  id: 'workers_ai',
  label: new TranslatableMarkup('Workers AI'),
)]
class WorkersAiProvider extends AiProviderClientBase implements
  ContainerFactoryPluginInterface,
  ChatInterface {

  use ChatTrait;

  /**
   * The OpenAI Client for API calls.
   *
   * @var \OpenAI\Client|null
   */
  protected $client;

  /**
   * API Key.
   *
   * @var string
   */
  protected string $apiKey = '';

  /**
   * API Key.
   *
   * @var string
   */
  protected string $cloudfareBaseURL = '';

  /**
   * API Key.
   *
   * @var string
   */
  protected string $cloudfareAccountId = '';

  /**
   * {@inheritdoc}
   */
  public function getConfiguredModels(?string $operation_type = NULL, array $capabilities = []): array {
    $cloudfareBaseURL = $this->getConfig()->get('cloudfare_base_url');
    $cloudfareAccountId = $this->getConfig()->get('cloudfare_account_id');
    $base_url = $cloudfareBaseURL . '/accounts/' . $cloudfareAccountId;
  
    if (in_array(AiModelCapability::ChatWithImageVision, $capabilities)) {
      return [];
    }
  
    try {
      $client = \Drupal::httpClient();
      $response = $client->request('GET', $base_url . "/ai/models/search", [
        'headers' => [
          'Authorization' => "Bearer pKum0EeqkErDqfIBwqDThOAkxRloUGHH5Wg7GMXI",
          'Content-Type' => 'application/json',
        ],
      ]);
      $body = $response->getBody()->getContents();
      $response_data = json_decode($body, TRUE);
    } catch (\Exception $e) {
      \Drupal::logger('workers_ai')->error('API request failed: @message', ['@message' => $e->getMessage()]);
      return [];
    }
  
    // Map raw task names to normalized operation types.
    $category_map = [
      'text generation' => 'chat',
      'text embeddings' => 'embeddings',
      'text classification' => 'moderation',
      'translation' => 'translate_text',
      'image classification' => 'image_classification',
      'object detection' => 'image_classification',
      'automatic speech recognition' => 'speech_to_text',
      'speech synthesis' => 'text_to_speech',
      'image-to-video generation' => 'image_to_video',
      'audio-to-audio generation' => 'audio_to_audio',
      'image and audio to video' => 'image_and_audio_to_video',
      'speech-to-speech' => 'speech_to_speech',
      'image to text' => 'chat_with_image_vision',
      'chat with json output' => 'chat_json_output',
    ];
  
    $models_by_category = [];
  
    if (!empty($response_data['result'])) {
      foreach ($response_data['result'] as $model) {
        $raw_category = strtolower($model['task']['name'] ?? 'unknown');
        $normalized_category = $category_map[$raw_category] ?? 'unknown';
  
        // Fallback: try to infer from model name
        if ($normalized_category === 'unknown') {
          $name = strtolower($model['name']);
          if (str_contains($name, 'embedding')) {
            $normalized_category = 'embeddings';
          } elseif (str_contains($name, 'llama')) {
            $normalized_category = 'chat_json_output';
          } elseif (str_contains($name, 'tts')) {
            $normalized_category = 'text_to_speech';
          } elseif (str_contains($name, 'asr')) {
            $normalized_category = 'speech_to_text';
          } elseif (str_contains($name, 'moderation')) {
            $normalized_category = 'moderation';
          } elseif (str_contains($name, 'image-classification')) {
            $normalized_category = 'image_classification';
          } elseif (str_contains($name, 'speech-to-speech')) {
            $normalized_category = 'speech_to_speech';
          } elseif (str_contains($name, 'audio-to-audio')) {
            $normalized_category = 'audio_to_audio';
          } elseif (str_contains($name, 'image-to-video')) {
            $normalized_category = 'image_to_video';
          }
  
          if ($normalized_category === 'unknown') {
            continue;
          }
        }
  
        $models_by_category[$normalized_category][$model['name']] = $model['name'];
      }
    }
  
    // Filter for ChatJsonOutput (LLaMA models only)
    if (in_array(AiModelCapability::ChatJsonOutput, $capabilities)) {
      foreach ($models_by_category as $cat => &$models) {
        foreach ($models as $id => $name) {
          if (strpos($id, 'llama') === FALSE) {
            unset($models[$id]);
          }
        }
        if (empty($models)) {
          unset($models_by_category[$cat]);
        }
      }
    }
  
    // If a specific operation_type is requested, return only those models.
    if ($operation_type !== NULL) {
      $operation_type = strtolower($operation_type);
      return $models_by_category[$operation_type] ?? [];
    }
  
    return $models_by_category;
  }  
  

  /**
   * {@inheritdoc}
   */
  public function isUsable(?string $operation_type = NULL, array $capabilities = []): bool {
    // No vision support.
    if (in_array(AiModelCapability::ChatWithImageVision, $capabilities)) {
      return FALSE;
    }
    // If its not configured, it is not usable.
    if (!$this->getConfig()->get('api_key')) {
      return FALSE;
    }
    else{
      $this->apiKey = $this->getConfig()->get('api_key');
    }
    if (!$this->getConfig()->get('cloudfare_base_url')) {
      return FALSE;
    }
    else{
      $this->cloudfareBaseURL = $this->getConfig()->get('cloudfare_base_url');
    }
    if (!$this->getConfig()->get('cloudfare_account_id')) {
      return FALSE;
    }
    else{ 
      $this->cloudfareAccountId = $this->getConfig()->get('cloudfare_account_id');
    }
    // If its one of the bundles that Workers AI supports its usable.
    if ($operation_type) {
      return in_array($operation_type, $this->getSupportedOperationTypes());
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedOperationTypes(): array {
    return [
      'chat',
      'embeddings',
      'speech_to_speech',
      'image_classification',
      'image_and_audio_to_video',
      'audio_to_audio',
      'speech_to_text',
      'text_to_speech',
      'image_to_video',
      'text_to_image',
      'translate_text',
      'moderation',
      'chat_with_image_vision',
      'chat_json_output',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(): ImmutableConfig {
    return $this->configFactory->get('workers_ai_provider.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function getApiDefinition(): array {
    // Load the configuration.
    $definition = Yaml::parseFile($this->moduleHandler->getModule('workers_ai_provider')->getPath() . '/definitions/api_defaults.yml');
    return $definition;
  }

  /**
   * {@inheritdoc}
   */
  public function getModelSettings(string $model_id, array $generalConfig = []): array {
    return $generalConfig;
  }

  /**
   * {@inheritdoc}
   */
  public function setAuthentication(mixed $authentication): void {
    // Set the new API key and reset the client.
    $this->apiKey = $authentication;
    $this->client = NULL;
  }

  /**
   * Gets the raw client.
   *
   * This is the client for inference.
   *
   * @return \OpenAI\Client
   *   The OpenAI client.
   */
  public function getClient(): Client {
    $this->loadClient();
    return $this->client;
  }

  /**
   * Loads the Workers AI Client with authentication if not initialized.
   */
  protected function loadClient(): void {
    if (!$this->client) {
      if (!$this->apiKey) {
        $this->setAuthentication($this->loadApiKey());
      }
      $cloudfareBaseURL = $this->getConfig()->get('cloudfare_base_url');
      $cloudfareAccountId = $this->getConfig()->get('cloudfare_account_id'); 
      $final_base_url = $cloudfareBaseURL .'/accounts/'. $cloudfareAccountId;
      $this->client = \OpenAI::factory()
        ->withApiKey($this->apiKey)
        ->withBaseUri($final_base_url)
        ->withHttpClient($this->httpClient)
        ->make();       
    }
  }

  /**
   * {@inheritdoc}
   */
  public function chat(array|string|ChatInput $input, string $model_id, array $tags = []): ChatOutput {
    $cloudfareBaseURL = $this->getConfig()->get('cloudfare_base_url');
    $cloudfareAccountId = $this->getConfig()->get('cloudfare_account_id');
    $base_url = $cloudfareBaseURL . '/accounts/' . $cloudfareAccountId;
    $chat_input = [];
  
    // Prepare chat messages.
    if ($input instanceof ChatInput) {
      if ($this->chatSystemRole) {
        $chat_input[] = [
          'role' => 'system',
          'content' => $this->chatSystemRole,
        ];
      }
      foreach ($input->getMessages() as $message) {
        $chat_input[] = [
          'role' => 'user',
          'content' => $message->getText(),
        ];
      }
    }
    // Build API request payload.
    $payload = [
      'messages' => $chat_input,
    ];
  
    try {
      $client = \Drupal::httpClient();
      $response = $client->post(
        $base_url."/ai/run/{$model_id}",
        [
          'headers' => [
            'Authorization' => 'Bearer pKum0EeqkErDqfIBwqDThOAkxRloUGHH5Wg7GMXI',
            'Content-Type' => 'application/json',
          ],
          'json' => $payload,
        ]
      );
  
      $data = json_decode($response->getBody()->getContents(), TRUE);
      $message_text = $data['result']['response'] ?? 'No response';  
      $message = new ChatMessage('assistant', $message_text);
      return new ChatOutput($message, $data, []);
    }
    catch (\Exception $e) {
      if (strpos($e->getMessage(), 'Rate limit') !== FALSE) {
        throw new AiRateLimitException($e->getMessage());
      }
      throw $e;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSetupData(): array {
    return [
      'key_config_name' => 'api_key',
      'default_models' => [
        'chat' => '@cf/meta/llama-3.3-70b-instruct-fp8-fast',
        'text_generation' => '@cf/meta/llama-3.3-70b-instruct-fp8-fast',
        'text_embeddings' => '@cf/meta/embeddings-001',
        'text_classification' => '@cf/meta/classify-001',
        'text_to_image' => '@cf/meta/imagegen-001',
        'summarization' => '@cf/meta/summarizer-001',
        'automatic_speech_recognition' => '@cf/meta/asr-001',
        'image_to_text' => '@cf/meta/imagetext-001',
        'translation' => '@cf/meta/translate-001',
        'image_classification' => '@cf/meta/imageclassify-001',
      ],
    ];
  }

}
